""" apu.datetime: anton python utils datetime module """

__version__ = (0, 0, 0)
__email__ = "anton.feldmann@gmail.com"
__author__ = "anton feldmann"

from apu.time.date_time import DateTime

__all__ = ['DateTime']
