""" apu.io anton python utils input output module """

__version__ = (0, 0, 0)
__email__ = "anton.feldmann@gmail.com"
__author__ = "anton feldmann"

from apu.io.dill import reconstruct, load

__all__ = ['reconstruct', "load"]
