""" apu.mp: anton python utils multiprocessing module """

__version__ = (0, 0, 0)
__email__ = "anton.feldmann@gmail.com"
__author__ = "anton feldmann"

from apu.mp.for_ import parallel_for

__all__ = ['parallel_for']
